
/******************************************************************************
* MODULE     : fromxml.cpp
* DESCRIPTION: conversion of logical xml and html trees into edit trees
* COPYRIGHT  : (C) 2003  Joris van der Hoeven, David Allouche
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "convert.hpp"
#include "scheme.hpp"

tree
xml_to_tree (string s) {
  return object_to_tree (call ("xml->object", s));
}

tree
xml_document_to_tree (string s) {
  if (!starts (s, "<?xml version=\"1.0\"?>\n\n<TeXmacs")) return "error";
  tree doc= as_scheme_tree (call ("xml->document", s));
  if ((!is_tuple (doc, "document")) || (N(doc)<2)) return "error";
  string version= "1.0.1.24";
  if (is_tuple (doc[1], "TeXmacs", 1) && is_atomic (doc[1][1]))
    version= doc[1][1]->label;
  doc= scheme_tree_to_tree (doc, get_codes (version));
  return upgrade (doc, version);
}

tree
html_to_tree (string s, string mode) {
  (void) mode;
  return object_to_tree (call ("html->tree", s));
}

tree
html_document_to_tree (string s, string& style) {
  tree res= as_scheme_tree (call ("html->document", s));
  style= as_string (res[1]);
  return scheme_tree_to_tree (res[0]);
}
